@@ -1,5 +1,5 @@
 /* JMenu.java --
-   Copyright (C) 2002, 2004, 2005  Free Software Foundation, Inc.
+   Copyright (C) 2002, 2004, 2005, 2006  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -39,19 +39,29 @@ exception statement from your version. */
 package javax.swing;
 
 import java.awt.Component;
+import java.awt.Dimension;
+import java.awt.GraphicsConfiguration;
+import java.awt.GraphicsDevice;
+import java.awt.GraphicsEnvironment;
+import java.awt.Insets;
 import java.awt.Point;
+import java.awt.Rectangle;
+import java.awt.Toolkit;
 import java.awt.event.KeyEvent;
 import java.awt.event.WindowAdapter;
 import java.awt.event.WindowEvent;
 import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
 import java.io.Serializable;
+import java.util.ArrayList;
 import java.util.EventListener;
 
 import javax.accessibility.Accessible;
 import javax.accessibility.AccessibleContext;
 import javax.accessibility.AccessibleRole;
 import javax.accessibility.AccessibleSelection;
+import javax.swing.event.ChangeEvent;
+import javax.swing.event.ChangeListener;
 import javax.swing.event.MenuEvent;
 import javax.swing.event.MenuListener;
 import javax.swing.plaf.MenuItemUI;
@@ -70,10 +80,40 @@ import javax.swing.plaf.MenuItemUI;
  */
 public class JMenu extends JMenuItem implements Accessible, MenuElement
 {
+  /**
+   * Receives notifications when the JMenu's ButtonModel is changed and
+   * fires menuSelected or menuDeselected events when appropriate.
+   */
+  private class MenuChangeListener
+    implements ChangeListener
+  {
+    /**
+     * Indicates the last selected state.
+     */
+    private boolean selected;
+
+    /**
+     * Receives notification when the JMenu's ButtonModel changes.
+     */
+    public void stateChanged(ChangeEvent ev)
+    {
+      ButtonModel m = (ButtonModel) ev.getSource();
+      boolean s = m.isSelected();
+      if (s != selected)
+        {
+          if (s)
+            fireMenuSelected();
+          else
+            fireMenuDeselected();
+          selected = s;
+        }
+    }
+  }
+
   private static final long serialVersionUID = 4227225638931828014L;
 
   /** A Popup menu associated with this menu, which pops up when menu is selected */
-  private JPopupMenu popupMenu = new JPopupMenu();
+  private JPopupMenu popupMenu = null;
 
   /** Whenever menu is selected or deselected the MenuEvent is fired to
      menu's registered listeners. */
@@ -86,10 +126,19 @@ public class JMenu extends JMenuItem implements Accessible, MenuElement
   /* PopupListener */
   protected WinListener popupListener;
 
-  /** Location at which popup menu associated with this menu will be
-     displayed */
+  /**
+   * Location at which popup menu associated with this menu will be
+   * displayed
+   */
   private Point menuLocation;
 
+  /**
+   * The ChangeListener for the ButtonModel.
+   *
+   * @see MenuChangeListener
+   */
+  private ChangeListener menuChangeListener;
+
   /**
    * Creates a new JMenu object.
    */
@@ -107,6 +156,7 @@ public class JMenu extends JMenuItem implements Accessible, MenuElement
   public JMenu(String text)
   {
     super(text);
+    popupMenu = new JPopupMenu(); 
     popupMenu.setInvoker(this);
     setOpaque(false);
   }
@@ -121,6 +171,7 @@ public class JMenu extends JMenuItem implements Accessible, MenuElement
   {
     super(action);
     createActionChangeListener(this);
+    popupMenu = new JPopupMenu();
     popupMenu.setInvoker(this);
     setOpaque(false);
   }
@@ -147,7 +198,7 @@ public class JMenu extends JMenuItem implements Accessible, MenuElement
    */
   public JMenuItem add(JMenuItem item)
   {
-    return popupMenu.add(item);
+    return getPopupMenu().add(item);
   }
 
   /**
@@ -159,7 +210,7 @@ public class JMenu extends JMenuItem implements Accessible, MenuElement
    */
   public Component add(Component component)
   {
-    popupMenu.insert(component, -1);
+    getPopupMenu().insert(component, -1);
     return component;
   }
 
@@ -173,7 +224,7 @@ public class JMenu extends JMenuItem implements Accessible, MenuElement
    */
   public Component add(Component component, int index)
   {
-    return popupMenu.add(component, index);
+    return getPopupMenu().add(component, index);
   }
 
   /**
@@ -185,7 +236,7 @@ public class JMenu extends JMenuItem implements Accessible, MenuElement
    */
   public JMenuItem add(String text)
   {
-    return popupMenu.add(text);
+    return add(new JMenuItem(text));
   }
 
   /**
@@ -197,7 +248,10 @@ public class JMenu extends JMenuItem implements Accessible, MenuElement
    */
   public JMenuItem add(Action action)
   {
-    return popupMenu.add(action);
+    JMenuItem i = createActionComponent(action);
+    i.setAction(action);
+    add(i);
+    return i;
   }
 
   /**
@@ -208,7 +262,7 @@ public class JMenu extends JMenuItem implements Accessible, MenuElement
    */
   public void remove(JMenuItem item)
   {
-    popupMenu.remove(item);
+    getPopupMenu().remove(item);
   }
 
   /**
@@ -218,7 +272,11 @@ public class JMenu extends JMenuItem implements Accessible, MenuElement
    */
   public void remove(int index)
   {
-    popupMenu.remove(index);
+    if (index < 0 || (index > 0 && getMenuComponentCount() == 0))
+      throw new IllegalArgumentException();
+  
+    if (getMenuComponentCount() > 0)
+      popupMenu.remove(index);
   }
 
   /**
@@ -228,8 +286,9 @@ public class JMenu extends JMenuItem implements Accessible, MenuElement
    */
   public void remove(Component component)
   {
-    int index = popupMenu.getComponentIndex(component);
-    popupMenu.remove(index);
+    int index = getPopupMenu().getComponentIndex(component);
+    if (index >= 0)
+      getPopupMenu().remove(index);
   }
 
   /**
@@ -237,7 +296,8 @@ public class JMenu extends JMenuItem implements Accessible, MenuElement
    */
   public void removeAll()
   {
-    popupMenu.removeAll();
+    if (popupMenu != null)
+      popupMenu.removeAll();
   }
 
   /**
@@ -266,7 +326,7 @@ public class JMenu extends JMenuItem implements Accessible, MenuElement
     if (index < 0)
       throw new IllegalArgumentException("index less than zero");
 
-    popupMenu.insert(item, index);
+    getPopupMenu().insert(item, index);
     return item;
   }
 
@@ -293,8 +353,7 @@ public class JMenu extends JMenuItem implements Accessible, MenuElement
    */
   public void updateUI()
   {
-    super.setUI((MenuItemUI) UIManager.getUI(this));
-    invalidate();
+    setUI((MenuItemUI) UIManager.getUI(this));
   }
 
   /**
@@ -315,7 +374,18 @@ public class JMenu extends JMenuItem implements Accessible, MenuElement
    */
   public void setModel(ButtonModel model)
   {
+    ButtonModel oldModel = getModel();
+    if (oldModel != null && menuChangeListener != null)
+      oldModel.removeChangeListener(menuChangeListener);
+
     super.setModel(model);
+
+    if (model != null)
+      {
+        if (menuChangeListener == null)
+          menuChangeListener = new MenuChangeListener();
+        model.addChangeListener(menuChangeListener);
+      }
   }
 
   /**
@@ -328,63 +398,6 @@ public class JMenu extends JMenuItem implements Accessible, MenuElement
     return super.isSelected();
   }
 
-  /**
-   * A helper method to handle setSelected calls from both mouse events and 
-   * direct calls to setSelected.  Direct calls shouldn't expand the popup
-   * menu and should select the JMenu even if it is disabled.  Mouse events
-   * only select the JMenu if it is enabled and should expand the popup menu
-   * associated with this JMenu.
-   * @param selected whether or not the JMenu was selected
-   * @param menuEnabled whether or not selecting the menu is "enabled".  This
-   * is always true for direct calls, and is set to isEnabled() for mouse 
-   * based calls.
-   * @param showMenu whether or not to show the popup menu
-   */
-  private void setSelectedHelper(boolean selected, boolean menuEnabled, boolean showMenu)
-  {
-    // If menu is selected and enabled, activates the menu and 
-    // displays associated popup.	
-    if (selected && menuEnabled)
-      {
-	super.setArmed(true);
-	super.setSelected(true);
-
-        // FIXME: The popup menu should be shown on the screen after certain
-        // number of seconds pass. The 'delay' property of this menu indicates
-        // this amount of seconds. 'delay' property is 0 by default.
-	if (isShowing())
-	  {
-	    fireMenuSelected();
-            
-	    int x = 0;
-	    int y = 0;
-            if (showMenu)
-              if (menuLocation == null)
-                {
-                  // Calculate correct position of the popup. Note that location of the popup 
-                  // passed to show() should be relative to the popup's invoker
-                  if (isTopLevelMenu())
-                    y = this.getHeight();
-                  else
-                    x = this.getWidth();
-                  getPopupMenu().show(this, x, y);
-                }
-              else
-                {
-                  getPopupMenu().show(this, menuLocation.x, menuLocation.y);
-                }
-	  }
-      }
-    
-    else
-      {
-	super.setSelected(false);
-	super.setArmed(false);
-	fireMenuDeselected();
-	popupMenu.setVisible(false);
-      }
-  }
-
   /**
    * Changes this menu selected state if selected is true and false otherwise
    * This method fires menuEvents to menu's registered listeners.
@@ -393,7 +406,9 @@ public class JMenu extends JMenuItem implements Accessible, MenuElement
    */
   public void setSelected(boolean selected)
   {
-    setSelectedHelper(selected, true, false); 
+    ButtonModel m = getModel();
+    if (selected != m.isSelected())
+      m.setSelected(selected);
   }
 
   /**
@@ -404,7 +419,7 @@ public class JMenu extends JMenuItem implements Accessible, MenuElement
    */
   public boolean isPopupMenuVisible()
   {
-    return popupMenu.isVisible();
+    return getPopupMenu().isVisible();
   }
 
   /**
@@ -414,23 +429,126 @@ public class JMenu extends JMenuItem implements Accessible, MenuElement
    */
   public void setPopupMenuVisible(boolean popup)
   {
-    if (getModel().isEnabled())
-      popupMenu.setVisible(popup);
+    if (popup != isPopupMenuVisible() && (isEnabled() || ! popup))
+      {
+        if (popup && isShowing())
+          {
+            // Set location as determined by getPopupLocation().
+            Point loc = menuLocation == null ? getPopupMenuOrigin()
+                                             : menuLocation;
+            getPopupMenu().show(this, loc.x, loc.y);
+          }
+        else
+          getPopupMenu().setVisible(false);
+      }
   }
 
   /**
-   * Returns origin point of the popup menu
+   * Returns origin point of the popup menu. This takes the screen bounds
+   * into account and places the popup where it fits best. 
    *
-   * @return Point containing
+   * @return the origin of the popup menu
    */
   protected Point getPopupMenuOrigin()
   {
-    // if menu in the menu bar
-    if (isTopLevelMenu())
-      return new Point(0, this.getHeight());
+    // The menu's screen location and size.
+    Point screenLoc = getLocationOnScreen();
+    Dimension size = getSize();
 
-    // if submenu            
-    return new Point(this.getWidth(), 0);
+    // Determine the popup's size.
+    JPopupMenu popup = getPopupMenu();
+    Dimension popupSize = popup.getSize();
+    if (popupSize.width == 0 || popupSize.height == 0)
+      popupSize = popup.getPreferredSize(); 
+
+    // Determine screen bounds.
+    Toolkit tk = Toolkit.getDefaultToolkit();
+    Rectangle screenBounds = new Rectangle(tk.getScreenSize());
+    GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
+    GraphicsDevice gd = ge.getDefaultScreenDevice();
+    GraphicsConfiguration gc = gd.getDefaultConfiguration();
+    Insets screenInsets = tk.getScreenInsets(gc);
+    screenBounds.x -= screenInsets.left;
+    screenBounds.width -= screenInsets.left + screenInsets.right;
+    screenBounds.y -= screenInsets.top;
+    screenBounds.height -= screenInsets.top + screenInsets.bottom;
+    screenLoc.x -= screenInsets.left;
+    screenLoc.y -= screenInsets.top;
+
+    Point point = new Point();
+    if (isTopLevelMenu())
+      {
+        // If menu in the menu bar.
+        int xOffset = UIManager.getInt("Menu.menuPopupOffsetX");
+        int yOffset = UIManager.getInt("Menu.menuPopupOffsetY");
+        // Determine X location.
+        if (getComponentOrientation().isLeftToRight())
+          {
+            // Prefer popup to the right.
+            point.x = xOffset;
+            // Check if it fits, otherwise place popup wherever it fits.
+            if (screenLoc.x + point.x + popupSize.width
+                > screenBounds.width + screenBounds.width
+                && screenBounds.width - size.width
+                   < 2 * (screenLoc.x - screenBounds.x))
+              // Popup to the right if there's not enough room.
+              point.x = size.width - xOffset - popupSize.width;
+          }
+        else
+          {
+            // Prefer popup to the left.
+            point.x = size.width - xOffset - popupSize.width;
+            if (screenLoc.x + point.x < screenBounds.x
+                && screenBounds.width - size.width
+                   > 2 * (screenLoc.x - screenBounds.x))
+              // Popup to the left if there's not enough room.
+              point.x = xOffset;
+          }
+        // Determine Y location. Prefer popping down.
+        point.y = size.height + yOffset;
+        if (screenLoc.y + point.y + popupSize.height >= screenBounds.height
+            && screenBounds.height - size.height
+               < 2 * (screenLoc.y - screenBounds.y))
+          // Position above if there's not enough room below.
+          point.y = - yOffset - popupSize.height;
+      }
+    else
+      {
+        // If submenu.
+        int xOffset = UIManager.getInt("Menu.submenuPopupOffsetX");
+        int yOffset = UIManager.getInt("Menu.submenuPopupOffsetY");
+        // Determine X location.
+        if (getComponentOrientation().isLeftToRight())
+          {
+            // Prefer popup to the right.
+            point.x = size.width + xOffset;
+            if (screenLoc.x + point.x + popupSize.width
+                >= screenBounds.x + screenBounds.width
+                && screenBounds.width - size.width
+                   < 2 * (screenLoc.x - screenBounds.x))
+              // Position to the left if there's not enough room on the right.
+              point.x = - xOffset - popupSize.width;
+          }
+        else
+          {
+            // Prefer popup on the left side.
+            point.x = - xOffset - popupSize.width;
+            if (screenLoc.x + point.x < screenBounds.x
+                && screenBounds.width - size.width
+                > 2 * (screenLoc.x - screenBounds.x))
+              // Popup to the right if there's not enough room.
+              point.x = size.width + xOffset;
+          }
+        // Determine Y location. Prefer popping down.
+        point.y = yOffset;
+        if (screenLoc.y + point.y + popupSize.height
+            >= screenBounds.y + screenBounds.height
+            && screenBounds.height - size.height
+            < 2 * (screenLoc.y - screenBounds.y))
+          // Pop up if there's not enough room below.
+          point.y = size.height - yOffset - popupSize.height;
+      }
+    return point;
   }
 
   /**
@@ -470,6 +588,8 @@ public class JMenu extends JMenuItem implements Accessible, MenuElement
   public void setMenuLocation(int x, int y)
   {
     menuLocation = new Point(x, y);
+    if (popupMenu != null)
+      popupMenu.setLocation(x, y);
   }
 
   /**
@@ -530,6 +650,9 @@ public class JMenu extends JMenuItem implements Accessible, MenuElement
     if (index < 0)
       throw new IllegalArgumentException("index less than 0");
 
+    if (getItemCount() == 0)
+      return null;
+    
     Component c = popupMenu.getComponentAtIndex(index);
 
     if (c instanceof JMenuItem)
@@ -558,7 +681,7 @@ public class JMenu extends JMenuItem implements Accessible, MenuElement
   public boolean isTearOff()
   {
     // NOT YET IMPLEMENTED 
-    return false;
+    throw new Error("The method isTearOff() has not yet been implemented.");
   }
 
   /**
@@ -568,7 +691,7 @@ public class JMenu extends JMenuItem implements Accessible, MenuElement
    */
   public int getMenuComponentCount()
   {
-    return popupMenu.getComponentCount();
+    return getPopupMenu().getComponentCount();
   }
 
   /**
@@ -581,7 +704,10 @@ public class JMenu extends JMenuItem implements Accessible, MenuElement
    */
   public Component getMenuComponent(int index)
   {
-    return (Component) popupMenu.getComponentAtIndex(index);
+    if (getPopupMenu() == null || getMenuComponentCount() == 0)
+      return null;
+    
+    return popupMenu.getComponentAtIndex(index);
   }
 
   /**
@@ -591,7 +717,7 @@ public class JMenu extends JMenuItem implements Accessible, MenuElement
    */
   public Component[] getMenuComponents()
   {
-    return popupMenu.getComponents();
+    return getPopupMenu().getComponents();
   }
 
   /**
@@ -626,6 +752,11 @@ public class JMenu extends JMenuItem implements Accessible, MenuElement
    */
   public JPopupMenu getPopupMenu()
   {
+    if (popupMenu == null)
+      {
+        popupMenu = new JPopupMenu();
+        popupMenu.setInvoker(this);
+      }
     return popupMenu;
   }
 
@@ -724,7 +855,7 @@ public class JMenu extends JMenuItem implements Accessible, MenuElement
   {
     // if this menu selection is true, then activate this menu and 
     // display popup associated with this menu
-    setSelectedHelper(changed, isEnabled(), true);
+    setSelected(changed);
   }
 
   /**
@@ -765,7 +896,7 @@ public class JMenu extends JMenuItem implements Accessible, MenuElement
    */
   protected void processKeyEvent(KeyEvent event)
   {
-    // TODO: Implement this properly.
+    MenuSelectionManager.defaultManager().processKeyEvent(event);
   }
 
   /**
@@ -810,7 +941,9 @@ public class JMenu extends JMenuItem implements Accessible, MenuElement
     return accessibleContext;
   }
 
-  // FIXME: This inner class is a complete stub and needs to be implemented.
+  /**
+   * Implements support for assisitive technologies for <code>JMenu</code>.
+   */
   protected class AccessibleJMenu extends AccessibleJMenuItem
     implements AccessibleSelection
   {
@@ -821,59 +954,245 @@ public class JMenu extends JMenuItem implements Accessible, MenuElement
       // Nothing to do here.
     }
 
+    /**
+     * Returns the number of accessible children of this object.
+     *
+     * @return the number of accessible children of this object
+     */
     public int getAccessibleChildrenCount()
     {
-      return 0;
+      Component[] children = getMenuComponents();
+      int count = 0;
+      for (int i = 0; i < children.length; i++)
+        {
+          if (children[i] instanceof Accessible)
+            count++;
+        }
+      return count;
     }
 
-    public Accessible getAccessibleChild(int value0)
+    /**
+     * Returns the accessible child with the specified <code>index</code>.
+     *
+     * @param index the index of the child to fetch
+     *
+     * @return the accessible child with the specified <code>index</code>
+     */
+    public Accessible getAccessibleChild(int index)
     {
-      return null;
+      Component[] children = getMenuComponents();
+      int count = 0;
+      Accessible found = null;
+      for (int i = 0; i < children.length; i++)
+        {
+          if (children[i] instanceof Accessible)
+            {
+              if (count == index)
+                {
+                  found = (Accessible) children[i];
+                  break;
+                }
+              count++;
+            }
+        }
+      return found;
     }
 
+    /**
+     * Returns the accessible selection of this object. AccessibleJMenus handle
+     * their selection themselves, so we always return <code>this</code> here.
+     *
+     * @return the accessible selection of this object
+     */
     public AccessibleSelection getAccessibleSelection()
     {
-      return null;
+      return this;
     }
 
-    public Accessible getAccessibleSelection(int value0)
+    /**
+     * Returns the selected accessible child with the specified
+     * <code>index</code>.
+     *
+     * @param index the index of the accessible selected child to return
+     *
+     * @return the selected accessible child with the specified
+     *         <code>index</code>
+     */
+    public Accessible getAccessibleSelection(int index)
     {
-      return null;
+      Accessible selected = null;
+      // Only one item can be selected, which must therefore have index == 0.
+      if (index == 0)
+        {
+          MenuSelectionManager msm = MenuSelectionManager.defaultManager();
+          MenuElement[] me = msm.getSelectedPath();
+          if (me != null)
+            {
+              for (int i = 0; i < me.length; i++)
+                {
+                  if (me[i] == JMenu.this)
+                    {
+                      // This JMenu is selected, find and return the next
+                      // JMenuItem in the path.
+                      do
+                        {
+                          if (me[i] instanceof Accessible)
+                            {
+                              selected = (Accessible) me[i];
+                              break;
+                            }
+                          i++;
+                        } while (i < me.length);
+                    }
+                  if (selected != null)
+                    break;
+                }
+            }
+        }
+      return selected;
     }
 
-    public boolean isAccessibleChildSelected(int value0)
+    /**
+     * Returns <code>true</code> if the accessible child with the specified
+     * index is selected, <code>false</code> otherwise.
+     *
+     * @param index the index of the accessible child to check
+     *
+     * @return <code>true</code> if the accessible child with the specified
+     *         index is selected, <code>false</code> otherwise
+     */
+    public boolean isAccessibleChildSelected(int index)
     {
-      return false;
+      boolean selected = false;
+      MenuSelectionManager msm = MenuSelectionManager.defaultManager();
+      MenuElement[] me = msm.getSelectedPath();
+      if (me != null)
+        {
+          Accessible toBeFound = getAccessibleChild(index);
+          for (int i = 0; i < me.length; i++)
+            {
+              if (me[i] == toBeFound)
+                {
+                  selected = true;
+                  break;
+                }
+            }
+        }
+      return selected;
     }
 
+    /**
+     * Returns the accessible role of this object, which is
+     * {@link AccessibleRole#MENU} for the AccessibleJMenu.
+     *
+     * @return the accessible role of this object
+     */
     public AccessibleRole getAccessibleRole()
     {
       return AccessibleRole.MENU;
     }
 
+    /**
+     * Returns the number of selected accessible children. This will be
+     * <code>0</code> if no item is selected, or <code>1</code> if an item
+     * is selected. AccessibleJMenu can have maximum 1 selected item.
+     *
+     * @return the number of selected accessible children
+     */
     public int getAccessibleSelectionCount()
     {
-      return 0;
+      int count = 0;
+      MenuSelectionManager msm = MenuSelectionManager.defaultManager();
+      MenuElement[] me = msm.getSelectedPath();
+      if (me != null)
+        {
+          for (int i = 0; i < me.length; i++)
+            {
+              if (me[i] == JMenu.this)
+                {
+                  if (i + 1 < me.length)
+                    {
+                      count = 1;
+                      break;
+                    }
+                }
+            }
+        }
+      return count;
     }
 
-    public void addAccessibleSelection(int value0)
+    /**
+     * Selects the accessible child with the specified index.
+     *
+     * @param index the index of the accessible child to select
+     */
+    public void addAccessibleSelection(int index)
     {
-      // TODO: Implement this properly.
+      Accessible child = getAccessibleChild(index);
+      if (child != null && child instanceof JMenuItem)
+        {
+          JMenuItem mi = (JMenuItem) child;
+          MenuSelectionManager msm = MenuSelectionManager.defaultManager();
+          msm.setSelectedPath(createPath(JMenu.this));
+        }
     }
 
-    public void removeAccessibleSelection(int value0)
+    /**
+     * Removes the item with the specified index from the selection.
+     *
+     * @param index the index of the selected item to remove from the selection
+     */
+    public void removeAccessibleSelection(int index)
     {
-      // TODO: Implement this properly.
+      Accessible child = getAccessibleChild(index);
+      if (child != null)
+        {
+          MenuSelectionManager msm = MenuSelectionManager.defaultManager();
+          MenuElement[] oldSelection = msm.getSelectedPath();
+          for (int i = 0; i < oldSelection.length; i++)
+            {
+              if (oldSelection[i] == child)
+                {
+                  // Found the specified child in the selection. Remove it
+                  // from the selection.
+                  MenuElement[] newSel = new MenuElement[i - 1];
+                  System.arraycopy(oldSelection, 0, newSel, 0, i - 1);
+                  msm.setSelectedPath(newSel);
+                  break;
+                }
+            }
+        }
     }
 
+    /**
+     * Removes all possibly selected accessible children of this object from
+     * the selection.
+     */
     public void clearAccessibleSelection()
     {
-      // TODO: Implement this properly.
+      MenuSelectionManager msm = MenuSelectionManager.defaultManager();
+      MenuElement[] oldSelection = msm.getSelectedPath();
+      for (int i = 0; i < oldSelection.length; i++)
+        {
+          if (oldSelection[i] == JMenu.this)
+            {
+              // Found this menu in the selection. Remove all children from
+              // the selection.
+              MenuElement[] newSel = new MenuElement[i];
+              System.arraycopy(oldSelection, 0, newSel, 0, i);
+              msm.setSelectedPath(newSel);
+              break;
+            }
+        }
     }
 
+    /**
+     * AccessibleJMenu don't support multiple selection, so this method
+     * does nothing.
+     */
     public void selectAllAccessibleSelection()
     {
-      // TODO: Implement this properly.
+      // Nothing to do here.
     }
   }
 
@@ -906,7 +1225,7 @@ public class JMenu extends JMenuItem implements Accessible, MenuElement
   /**
    * This class listens to PropertyChangeEvents occuring in menu's action
    */
-  protected class ActionChangedListener implements PropertyChangeListener
+  private class ActionChangedListener implements PropertyChangeListener
   {
     /** menu item associated with the action */
     private JMenuItem menuItem;
@@ -928,4 +1247,44 @@ public class JMenu extends JMenuItem implements Accessible, MenuElement
     }
   }
 
+  /**
+   * Creates an array to be feeded as argument to
+   * {@link MenuSelectionManager#setSelectedPath(MenuElement[])} for the
+   * specified element. This has the effect of selecting the specified element
+   * and all its parents.
+   *
+   * @param leaf the leaf element for which to create the selected path
+   *
+   * @return the selected path array
+   */
+  MenuElement[] createPath(JMenu leaf)
+  {
+    ArrayList path = new ArrayList();
+    MenuElement[] array = null;
+    Component current = leaf.getPopupMenu();
+    while (true)
+      {
+        if (current instanceof JPopupMenu)
+          {
+            JPopupMenu popupMenu = (JPopupMenu) current;
+            path.add(0, popupMenu);
+            current = popupMenu.getInvoker();
+          }
+        else if (current instanceof JMenu)
+          {
+            JMenu menu = (JMenu) current;
+            path.add(0, menu);
+            current = menu.getParent();
+          }
+        else if (current instanceof JMenuBar)
+          {
+            JMenuBar menuBar = (JMenuBar) current;
+            path.add(0, menuBar);
+            array = new MenuElement[path.size()];
+            array = (MenuElement[]) path.toArray(array);
+            break;
+          }
+      }
+    return array;
+  }
 }
